local enabled = 1
local fishing = nil
local formingRaid = nil
local f2piGroup = nil
local askers = {}
local passLeadTo

local f = CreateFrame("frame")

f:RegisterEvent("CHAT_MSG_WHISPER")
f:RegisterEvent("PARTY_MEMBERS_CHANGED")
if not IsTrialAccount() then
	DEFAULT_CHAT_FRAME:AddMessage("|cFFFF00FFF2PI version "..GetAddOnMetadata("F2PI", "Version").." loaded.\nType /f2pi in chat to toggle off/on (will automatically be disabled when using LFD/BGs).|r")
else
	DEFAULT_CHAT_FRAME:AddMessage("|cFFFF00FFF2PI version "..GetAddOnMetadata("F2PI", "Version").." loaded.\nThe addon has been disabled as you are on a trial account, and cannot invite.|r")
	enabled = nil
end
local function F2PI_FilterOutgoing(self, event, ...)
	local msg = ...
	if msg:match("%[F2PI%]") then
		return true
	else
		return false
	end
end

local function F2PI_FilterIncoming(self, event, ...)
	local msg = ...
	if msg:match("^inv$") or msg:match("^lead$") or msg:match("^raid$") or msg:match("^close$") or msg:match("%[F2PI%]") then
		return true
	else
		return false
	end
end

ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER_INFORM", F2PI_FilterOutgoing)
ChatFrame_AddMessageEventFilter("CHAT_MSG_PARTY", F2PI_FilterOutgoing)
ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER", F2PI_FilterIncoming)

f:SetScript("OnEvent", function(self, event, ...)
	if event == ("CHAT_MSG_WHISPER") then
		local msg, sender = ...
		if (not F2PChat_IsBlocked(sender, 15)) then
			if (enabled) then
			
				--find out if you are either queued for BGs or LFD, or are in a BG/dungeon after queueing.
				local busy = false
				if (GetLFGMode() ~= nil) and (GetLFGMode() ~= "abandonedInDungeon") then
					busy = true
				end
				for x = 1, GetMaxBattlefieldID() do
					if GetBattlefieldStatus(x) ~= "none" then
						busy = true
					end
				end
		
				if not msg:match("\[F2PI\]") then --stop the addons getting into a loop of messaging each other
					-- 'inv' or 'lead' was in the message, but was not it's only content; or the word group was used, indicating that the sender might have asked for a group.
					if ((msg:lower():match("inv")) and (not msg:lower():match("^inv$"))) or (msg:lower():match("group")) then
						SendChatMessage("[F2PI] If you are trying form a group, the command to whisper to the addon is 'inv'.", "WHISPER", nil, sender)

					-- 'inv' command received
					elseif msg:lower():match("^inv$") then
						--not using BG/LFD, not yet in a party, or is leader of a party
						if (not busy) and ((not UnitExists("party1")) or IsPartyLeader("player")) then
							--switch over to a raid to make space for additional members if we're at the party size limit
							if (GetNumPartyMembers() == 4) and (not UnitInRaid("player")) then
								ConvertToRaid()
							end
							tinsert(askers, sender)
							InviteUnit(sender)
							if (not fishing) then
								SendChatMessage("[F2PI] The first person to join will be given lead of the group when it is filled, or when you send 'close' to signal that all members have joined, and that I should drop.", "WHISPER", nil, sender)
								SendChatMessage("[F2PI] If you wish to change to another leader, then they must send the command 'lead'.", "WHISPER", nil, sender)
								SendChatMessage("[F2PI] If you want to have more than 5 people in your group, then you must send the command 'raid' to stop me from dropping group when I have invited 5 people.", "WHISPER", nil, sender)
							elseif (fishing) then
								SendChatMessage("[F2PI] Fishing Competition mode is active.  Leadership will not be passed.", "WHISPER", nil, sender)
							end
						elseif ( busy or (UnitExists("party1") and not IsPartyLeader("player")) ) then
							SendChatMessage("[F2PI] I'm doing dungeons or BGs right now, so I'm not available to invite.", "WHISPER", nil, sender)
						end

					--'lead' command received
					elseif msg:lower():match("^lead$") and (not fishing) and UnitInParty(sender) then
						if (not busy) and UnitExists("party1") and IsPartyLeader("player") and UnitInParty(sender) then
							passLeadTo = sender
							SendChatMessage("[F2PI] "..sender.." will now be leading this party when I drop.  Send 'close' when everyone has joined to ask me to do that.", "PARTY", nil, sender)
						elseif (not busy) and not (UnitExists("party1") or UnitInParty(sender)) then
							SendChatMessage("[F2PI] I am not leading your party.", "WHISPER", nil, sender)
						end
						
					--'raid' command received
					elseif msg:lower():match("^raid$") and (not fishing) and UnitInParty(sender) then
						formingRaid = 1
						SendChatMessage("[F2PI] Party size limit removed.  This group will now allow more than 5 people to join.", "PARTY", nil, sender)
						
					--'close' command received
					elseif msg:lower():match("^close$") and (not fishing) and UnitInParty(sender) then
						F2PI_CloseGroup(passLeadTo)
					end
				end
			elseif (not enabled) and (not msg:match("\[F2PI\]")) and ( (msg:lower():match("inv")) )  then
				SendChatMessage("[F2PI] My invite addon is currently disabled.", "WHISPER", nil, sender)
			end
		else
			SendChatMessage("[F2PI] You must reach level 15 to be eligible for invites.", "WHISPER", nil, sender)
		end
		
	elseif event == ("PARTY_MEMBERS_CHANGED") then
		
		--if we have reached the party limit, and no-one has asked for a raid sized group, pass lead and leave the group
		if (f2piGroup == 1) and (GetNumRaidMembers() == 6) and (not formingRaid) and (not fishing) then
			F2PI_CloseGroup(passLeadTo)
		
		--if anyone asked for a group
		elseif #askers > 0 then
			--check if the person who joined is one of them
			for x = 1, #askers do
				if UnitInParty(askers[x]) then
					f2piGroup = 1  --mark this as an F2PI formed group if so
				end
			end
			
			--if it is...
			if f2piGroup then
				--and they're the first to join...
				if (GetNumPartyMembers() == 1) then
					passLeadTo = UnitName("party1")  --well be passing lead to them when the group closes
				end
			end

		--if you're no longer in a party, reset the function's flags
		elseif (not UnitExists("party1")) then
			fishing = nil
			formingRaid = nil
			f2piGroup = nil
			askers = {}
			passLeadTo = nil
		end
	end
end
)

function F2PI_CloseGroup(newLead)
	if (GetNumRaidMembers() == 6) then
		SendChatMessage("[F2PI] You will need to convert from raid to party before queueing for LFD.  BGs may still be queued as a raid of 5 though.", "RAID", nil)
	end
	local partyList = {}
	if UnitInRaid("player") then
		for x = 1, GetNumRaidMembers() do
			local y = GetRaidRosterInfo(x)
			if y ~= UnitName("player") then
				if y == newLead then
					y = y.." (leader)"
				end
				tinsert(partyList, y)
			end
		end
	elseif not UnitInRaid("player") then
		for x = 1, GetNumPartyMembers() do
			local y = UnitName("party"..x)
			if y ~= UnitName("player") then
				if y == newLead then
					y = y.." (leader)"
				end
				tinsert(partyList, y)
			end
		end
	end
	local msg = "[F2PI] Group formed: "..table.concat(partyList, ", ")
	F2PChat_RouteChatMessage(msg)  --send via addon
	F2PChat_FormatIncoming(msg, F2PAddonGlobalVars.myName) --print sent message
	PromoteToLeader(newLead, 1)
	LeaveParty()
end

local commandTable = {
	["fish"] = function()	
		if fishing == 1 then
			fishing = nil
			print("[F2PI] fishing competition mode disabled")
		else
			fishing = 1
			print("[F2PI] fishing competition mode enabled")
		end
	end,
	["default"] = function()
		if enabled == 1 then
			enabled = nil
			print("[F2PI] disabled")
		else
			enabled = 1
			print("[F2PI] enabled")
		end
	end,
}

local function runSlashCommand(message, cmdTable)
	local command, parameters = string.split(" ", message, 2)
	if (command == nil) or (command == "") then
		commandTable.default()
	elseif command == "fish" then
		commandTable.fish()
	end
end

SLASH_F2PI1 = "/f2pi"
SlashCmdList["F2PI"] = function(txt)
	runSlashCommand(txt, commandTable)
end